"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.make = exports.layer = void 0;
var Error = _interopRequireWildcard(require("@effect/platform/Error"));
var Terminal = _interopRequireWildcard(require("@effect/platform/Terminal"));
var Effect = _interopRequireWildcard(require("effect/Effect"));
var Layer = _interopRequireWildcard(require("effect/Layer"));
var Option = _interopRequireWildcard(require("effect/Option"));
var readline = _interopRequireWildcard(require("node:readline"));
function _interopRequireWildcard(e, t) { if ("function" == typeof WeakMap) var r = new WeakMap(), n = new WeakMap(); return (_interopRequireWildcard = function (e, t) { if (!t && e && e.__esModule) return e; var o, i, f = { __proto__: null, default: e }; if (null === e || "object" != typeof e && "function" != typeof e) return f; if (o = t ? n : r) { if (o.has(e)) return o.get(e); o.set(e, f); } for (const t in e) "default" !== t && {}.hasOwnProperty.call(e, t) && ((i = (o = Object.defineProperty) && Object.getOwnPropertyDescriptor(e, t)) && (i.get || i.set) ? o(f, t, i) : f[t] = e[t]); return f; })(e, t); }
const defaultShouldQuit = input => input.key.ctrl && (input.key.name === "c" || input.key.name === "d");
/** @internal */
const make = (shouldQuit = defaultShouldQuit) => Effect.gen(function* () {
  const input = yield* Effect.sync(() => globalThis.process.stdin);
  const output = yield* Effect.sync(() => globalThis.process.stdout);
  // Acquire a readline interface
  const acquireReadlineInterface = Effect.sync(() => readline.createInterface({
    input,
    escapeCodeTimeout: 50
  }));
  // Uses the readline interface to force `stdin` to emit keypress events
  const emitKeypressEvents = rl => {
    readline.emitKeypressEvents(input, rl);
    if (input.isTTY) {
      input.setRawMode(true);
    }
    return rl;
  };
  // Close the `readline` interface
  const releaseReadlineInterface = rl => Effect.sync(() => {
    if (input.isTTY) {
      input.setRawMode(false);
    }
    rl.close();
  });
  // Handle the `"keypress"` event emitted by `stdin` (forced by readline)
  const handleKeypressEvent = input => Effect.async(resume => {
    const handleKeypress = (input, key) => {
      const userInput = {
        input: Option.fromNullable(input),
        key: {
          name: key.name || "",
          ctrl: key.ctrl || false,
          meta: key.meta || false,
          shift: key.shift || false
        }
      };
      if (shouldQuit(userInput)) {
        resume(Effect.fail(new Terminal.QuitException()));
      } else {
        resume(Effect.succeed(userInput));
      }
    };
    input.once("keypress", handleKeypress);
    return Effect.sync(() => {
      input.removeListener("keypress", handleKeypress);
    });
  });
  // Handle the `"line"` event emitted by the readline interface
  const handleLineEvent = rl => Effect.async(resume => {
    const handleLine = line => {
      resume(Effect.succeed(line));
    };
    rl.on("line", handleLine);
    return Effect.sync(() => {
      rl.removeListener("line", handleLine);
    });
  });
  const readInput = Effect.acquireUseRelease(acquireReadlineInterface.pipe(Effect.map(emitKeypressEvents)), () => handleKeypressEvent(input), releaseReadlineInterface);
  const readLine = Effect.acquireUseRelease(acquireReadlineInterface, rl => handleLineEvent(rl), releaseReadlineInterface);
  const display = prompt => Effect.uninterruptible(Effect.async(resume => {
    output.write(prompt, err => {
      if (err) {
        resume(Effect.fail(new Error.BadArgument({
          module: "Terminal",
          method: "display",
          description: "Failed to write prompt to output",
          cause: err
        })));
      }
      resume(Effect.void);
    });
  }));
  return Terminal.Terminal.of({
    // The columns property can be undefined if stdout was redirected
    columns: Effect.sync(() => output.columns || 0),
    readInput,
    readLine,
    display
  });
});
/** @internal */
exports.make = make;
const layer = exports.layer = /*#__PURE__*/Layer.scoped(Terminal.Terminal, /*#__PURE__*/make(defaultShouldQuit));
//# sourceMappingURL=terminal.js.map